@@ -1,5 +1,7 @@
 """Koninklijke Philips N.V., 2019 - 2020. All rights reserved."""
 from __future__ import print_function
+
+import csv
 import sys
 import subprocess
 import shutil
@@ -7,17 +9,19 @@ import os
 import json
 import xml.etree.ElementTree as ETree
 import argparse
-from guardrail_globals import GuardrailGlobals #pylint: disable=E0401
-import guardrails_log as cl #pylint: disable=E0401
+
+from guardrail_globals import GuardrailGlobals  # pylint: disable=E0401
+import guardrails_log as cl  # pylint: disable=E0401
 
 LOG = cl.get_logger()
 
 
 def create_parser(args):
-    """ Function which add the command line arguments required for the commandline input
-    of guardrails"""
+    """ Function which add the command line arguments required for
+     the commandline input of guardrails"""
     # Create the parser
-    func_parser = argparse.ArgumentParser(description='Guardrails for python programs')
+    func_parser = argparse.ArgumentParser(
+        description='Guardrails for python programs')
 
     # Add the arguments
     func_parser.add_argument('--path',
@@ -43,10 +47,11 @@ class Guardails(GuardrailGlobals):
           path_ini (string): The path to guardrail.ini file.
 
         """
-        super(Guardails, self).__init__()
+        if sys.version_info >= (3, 5, 7):
+            super(Guardails, self).__init__()  # python 2.7 does not support
         self.set_all(path_ini)
 
-    def call_subprocess(self, cmd):  #pylint: disable=R0201
+    def call_subprocess(self, cmd):  # pylint: disable=R0201
         """
         Function to call subprocess to issue system commands
 
@@ -73,7 +78,8 @@ class Guardails(GuardrailGlobals):
         if folders is None:
             folders = self.all_folders
         list_to_str = ''.join([str(elem) for elem in folders])
-        LOG.info("list_to_str_folders function returned:%s", str(list_to_str))  # pragma: no mutate
+        LOG.info("list_to_str_folders function returned:%s",
+                 str(list_to_str))  # pragma: no mutate
         return list_to_str
 
     @staticmethod
@@ -112,7 +118,8 @@ class Guardails(GuardrailGlobals):
         """
         if not os.path.exists(file_path):
             LOG.info("jscpd report not generated")  # pragma: no mutate
-            print("\n\nGuardrail {}, failed {}.\n".format("gating", "jscpd"))  # pragma: no mutate
+            print("\n\nGuardrail {}, failed {}.\n".
+                  format("gating", "jscpd"))  # pragma: no mutate
             sys.exit(1)
         else:
             return
@@ -122,8 +129,9 @@ class Guardails(GuardrailGlobals):
         Function to judge if JSCPD gating pass/fail
 
         Parameters:
-          duplicate_limit (int): allowed duplicate limit for the jscpd/copy paste detection
-          json_file (string): jscpd report file
+          duplicate_limit (int): allowed duplicate limit for the jscpd/copy
+           paste detection json_file (string): jscpd report file
+           json_file: input data file
 
         Returns:
         return value. None
@@ -136,23 +144,28 @@ class Guardails(GuardrailGlobals):
                 lines = int(data['statistics']['total']['lines'])
             if lines > 0:
                 if per <= float(duplicate_limit):
-                    LOG.info("Guardrail gating passed jscpd")  # pragma: no mutate
+                    LOG.info("Guardrail gating passed"
+                             " jscpd")  # pragma: no mutate
 
                 else:
-                    LOG.info("Guardrail gating, failed jscpd.")  # pragma: no mutate
+                    LOG.info("Guardrail gating,"
+                             " failed jscpd.")  # pragma: no mutate
                     sys.exit(1)
             else:
-                LOG.info("jscpd report not correctly generated")  # pragma: no mutate
+                LOG.info("jscpd report not correctly"
+                         " generated")  # pragma: no mutate
                 sys.exit(1)
         except KeyError:
-            LOG.info("jscpd report not generated, key is not found in json")  # pragma: no mutate
+            LOG.info("jscpd report not generated, key is not"
+                     " found in json")  # pragma: no mutate
             sys.exit(1)
 
     @staticmethod
     def check_pass_fail(failure, total, allow_fail_per):
         """
         Function which check the failure percentile against total mutation runs
-        Exits successfully if actual failure is equal or less than the specified allowed failure
+        Exits successfully if actual failure is equal or less than the
+         specified allowed failure
 
         Parameters:
           failure (int): is the total of 'disabled + errors + failures'
@@ -165,18 +178,22 @@ class Guardails(GuardrailGlobals):
         per = int(total) * (allow_fail_per / 100)
         if total:
             if failure <= per:
-                LOG.info("Guardrail gating, passed mutation.")  # pragma: no mutate
+                LOG.info("Guardrail gating,"
+                         " passed mutation.")  # pragma: no mutate
             else:
-                LOG.info("Guardrail gating, failed mutation.")  # pragma: no mutate
+                LOG.info("Guardrail gating,"
+                         " failed mutation.")  # pragma: no mutate
                 sys.exit(1)
         else:
-            LOG.info("Guardrail gating, failed: mutation test did not run.")  # pragma: no mutate
+            LOG.info("Guardrail gating, failed: mutation"
+                     " test did not run.")  # pragma: no mutate
             sys.exit(1)
 
     @staticmethod
     def get_index_cnn(root):
         """
-        Function to parse the xml file generated by lizard for cyclomatic complexity to identify the index of CNN
+        Function to parse the xml file generated by lizard for cyclomatic
+         complexity to identify the index of CNN
 
         Parameters:
           root (etree node): root node of the parsed xml
@@ -189,18 +206,20 @@ class Guardails(GuardrailGlobals):
         try:
             labels = root.find('labels')
             for label in labels.iter('label'):
-                if str(label.text).upper() == str("CCN"):
+                if str(label.text).strip().upper() == str("CCN"):
                     val = count
                 else:
                     count += 1
             return val
         except AttributeError:
-            LOG.info("Guardrail unable to find the tag CCN in the report ")  # pragma: no mutate
+            LOG.info("Guardrail unable to find the tag CCN"
+                     " in the report ")  # pragma: no mutate
             sys.exit(1)
 
     def get_all_func_cnn(self, root):
         """
-        Function which create a dictionary with all functions in the parsed files/source with their CNN
+        Function which create a dictionary with all functions in the parsed
+         files/source with their CNN
 
         Parameters:
           root (etree node): root node of the parsed xml
@@ -220,29 +239,34 @@ class Guardails(GuardrailGlobals):
                 cyclo_dict[str(item.attrib['name'])] = temp_val[index].text
                 temp_val = []
             if not cyclo_dict:
-                LOG.info("Guardrail unable to find the tags item/value/name in the report file ")  # pragma: no mutate
+                LOG.info("Guardrail unable to find the tags item/value/name"
+                         " in the report file ")  # pragma: no mutate
                 sys.exit(1)
             else:
                 return cyclo_dict
         except KeyError:
-            LOG.info("Guardrail unable to find the tags item/value/name in the report ")  # pragma: no mutate
+            LOG.info("Guardrail unable to find the tags"
+                     " item/value/name in the report ")  # pragma: no mutate
             sys.exit(1)
 
     def parse_cyclo_report_xml(self, xml_file):
-        """ Function usd to fetch the necessary data from the xml output - lizard
+        """ Function usd to fetch the necessary data from the xml
+         output - lizard
 
         Parameters:
           xml_file (string): path to the xml file to be parsed
 
         Returns:
-        (dictionary) with list of function names and its Cyclomatic complexity or None
+        (dictionary) with list of function names and its
+         Cyclomatic complexity or None
 
         """
         try:
             root = ETree.parse(xml_file).getroot()
             for functions in root.iter('measure'):
                 if functions.attrib['type'] == "Function":
-                    LOG.info("successfully found functions with CNN")  # pragma: no mutate
+                    LOG.info("successfully found functions"
+                             " with CNN")  # pragma: no mutate
                     return self.get_all_func_cnn(functions)
             return None
 
@@ -250,11 +274,13 @@ class Guardails(GuardrailGlobals):
             LOG.info("cc.xml report file path")  # pragma: no mutate
             sys.exit(1)
         except KeyError:
-            LOG.info("tags required are not found in cc.xml report file path")  # pragma: no mutate
+            LOG.info("tags required are not found in cc.xml"
+                     " report file path")  # pragma: no mutate
             sys.exit(1)
 
     def parse_mutmut_report_xml(self, allow_fail, xml_file):
-        """ Function usd to fetch the necessary data from the xml output - mutmut
+        """ Function usd to fetch the necessary data from the
+         xml output - mutmut
 
         Parameters:
           xml_file (string): path to the xml file to be parsed
@@ -273,54 +299,78 @@ class Guardails(GuardrailGlobals):
             total_fail = disabled + errors + failures
             self.check_pass_fail(total_fail, tests, allow_fail)
         except IOError:
-            LOG.info("mutmut.xml report file path cound not be found")  # pragma: no mutate
+            LOG.info("mutmut.xml report file path cound"
+                     " not be found")  # pragma: no mutate
             sys.exit(1)
 
     def guardrail_lint(self):
-        """ Function which run the linting check (Static analysis for the files"""
+        """ Function which run the linting check
+         (Static analysis for the files"""
         LOG.info("Started Linting gate")  # pragma: no mutate
-        retval = self.call_subprocess("%s %s >%s" % (self.generate_pylint_cmd(), self.list_to_str_folders(),
-                                                     os.path.join(self.report_folder, "linting_Report.txt")))
+        open(os.path.join(self.report_folder, "linting_Report.txt"), "w")
+        cmd_list = self.generate_pylint_cmd()
+        retval = (self.call_subprocess("%s >%s" %
+                                       (cmd_list,
+                                        os.path.join(
+                                            self.report_folder,
+                                            "linting_Report.txt"))))
         self.validate_return(retval, "Linting", True)
         print("Passed linting gate")  # pragma: no mutate
         print('====================================')  # pragma: no mutate
 
     def guardrail_jscpd(self):
-        """ Function which conduct copy past detection for the folders specified"""
+        """ Function which conduct copy past detection
+         for the folders specified"""
         LOG.info("Started jscpd gate")  # pragma: no mutate
-        retval = self.call_subprocess('jscpd --min-tokens %s  %s  --max-lines 100000 --max-size 100mb --reporters '
-                                      '"json" --mode "strict" %s -o %s %s' %
-                                      (self.dup_token, self.jscpd_ignore_file(), self.jscpd_format(),
-                                       self.report_folder, self.list_to_str_folders()))
-        self.validate_return(retval, "Copy Paste Detection report generation ", False)
-        self.parse_jscpd_report_json(self.allow_dup, os.path.join(self.report_folder, "jscpd-report.json"))
+        retval = self.call_subprocess('jscpd --min-tokens %s  %s  --max-lines'
+                                      ' 100000 --max-size 100mb --reporters '
+                                      '"json,html" --mode "strict" %s -o %s %s'
+                                      % (self.dup_token,
+                                         self.jscpd_ignore_file(),
+                                         self.jscpd_format(),
+                                         self.report_folder,
+                                         self.jscpd_root))
+        self.validate_return(retval, "Copy Paste Detection report"
+                                     " generation ", False)
+        self.parse_jscpd_report_json(self.allow_dup, os.path.join(
+            self.report_folder, "jscpd-report.json"))
         print("Passed JSCPD gating")  # pragma: no mutate
         print('====================================')  # pragma: no mutate
 
     def guardrail_test(self):
-        """ Function which conduct testing using pytest for the folders specified"""
+        """ Function which conduct testing using pytest for the
+         folders specified"""
         LOG.info("Started test gate")  # pragma: no mutate
-        retval = self.call_subprocess('%s -m pytest %s %s --cov-report "html" --cov=%s' %
-                                      (self.python, self.pytest, self.cov_rc_file(), self.list_to_str_folders(
-                                          self.src_folder)))
-        self.validate_return(retval, "Test execution and coverage generation", False)
+        retval = self.call_subprocess('%s -m pytest %s %s --cov-report'
+                                      ' "html" --cov=%s' %
+                                      (self.python,
+                                       self.pytest,
+                                       self.cov_rc_file(),
+                                       self.list_to_str_folders(
+                                           self.src_folder)))
+        self.validate_return(retval, "Test execution and coverage"
+                                     " generation", False)
         print("Passed testing using pytest")  # pragma: no mutate
         print('====================================')  # pragma: no mutate
 
     def guardrail_coverage(self):
         """
-        Function which conduct coverage based on the test executed using pytest for the folders
-        specified
+        Function which conduct coverage based on the test executed using
+         pytest for the folders specified
         """
         LOG.info("Started coverage gate")  # pragma: no mutate
-        retval = self.call_subprocess('%s -m coverage report --fail-under=%s' % (self.python, self.percent_cov))
+        retval = self.call_subprocess('%s -m coverage report'
+                                      ' --fail-under=%s' % (self.python,
+                                                            self.percent_cov))
+        self.mov_cov_report()
         self.validate_return(retval, "Coverage threshold", True)
         print("Passed test coverage gating")  # pragma: no mutate
         print('====================================')  # pragma: no mutate
 
     def mov_cov_report(self):
         """
-        Function which is used to move the coverage report folder from where it generated to specified report folder
+        Function which is used to move the coverage report folder from where
+         it generated to specified report folder
         """
         LOG.info("Started moving coverage file")  # pragma: no mutate
         dirpath = os.path.join(self.report_folder, "coverage_Report")
@@ -331,43 +381,69 @@ class Guardails(GuardrailGlobals):
             shutil.move(src, dirpath)
 
     def guardrail_mutation(self):
-        """ Function which is used to conduct the mutation testing and gate it with allowed mutants % """
-
+        """ Function which is used to conduct the mutation testing and
+         gate it with allowed mutants % """
         LOG.info("Started mutation gate")  # pragma: no mutate
-        retval = self.call_subprocess('%s -m mutmut --paths-to-mutate %s run || true' % (self.python, os.path.join(
-            self.list_to_str_folders(self.src_folder))))
+        retval = self.call_subprocess('%s -m mutmut --paths-to-mutate %s'
+                                      ' run || true' %
+                                      (self.python,
+                                       os.path.join(
+                                           self.list_to_str_folders(
+                                               self.src_folder))))
         self.validate_return(retval, "Mutation testing ", False)
         retval = self.call_subprocess(
-            'python -m mutmut junitxml --suspicious-policy=ignore --untested-policy=ignore > %s' % (
+            'python -m mutmut junitxml --suspicious-policy=ignore'
+            ' --untested-policy=ignore > %s' % (
                 os.path.join(self.report_folder, 'mutmut.xml')))
-        self.validate_return(retval, "Mutation testing report generation", False)
-        self.parse_mutmut_report_xml(self.allow_mutants, os.path.join(self.report_folder, 'mutmut.xml'))
+        self.validate_return(retval, "Mutation testing report"
+                                     " generation", False)
+        self.parse_mutmut_report_xml(self.allow_mutants, os.path.join(
+            self.report_folder, 'mutmut.xml'))
         print("Passed mutation testing gate")  # pragma: no mutate
         print('====================================')  # pragma: no mutate
 
     def guardrail_cyclomatic_complexity(self):
         """
-        Function which is used to conduct the cyclomatic complexity and gate it with allowed cyclomatic
-        complexity value
+        Function which is used to conduct the cyclomatic complexity
+         and gate it with allowed cyclomatic complexity value
         """
         LOG.info("Started cyclomatic complexity gate")  # pragma: no mutate
-        retval = self.call_subprocess('%s -m lizard %s %s -X > %s' % (self.python, os.path.join(
-            self.list_to_str_folders(self.src_folder)), self.get_exclude_cc(),
-                                                                      os.path.join(self.report_folder, 'CC.xml')))
-        self.validate_return(retval, "Cyclomating complexity generation ", False)
-        complexity = self.parse_cyclo_report_xml(os.path.join(self.report_folder, 'CC.xml'))
-        cyclo_complex = [cyclo_complex for cyclo_complex in complexity if int(complexity.get(cyclo_complex)) >
-                         self.cc_limit]
+        retval = self.call_subprocess('%s -m lizard %s %s -X > %s' %
+                                      (self.python, os.path.join(
+                                          self.list_to_str_folders(
+                                              self.src_folder)),
+                                       self.get_exclude_cc(),
+                                       os.path.join(self.report_folder,
+                                                    'CC.xml')))
+        self.validate_return(retval, "Cyclomating complexity"
+                                     " generation ", False)
+        complexity = self.parse_cyclo_report_xml(os.path.join(
+            self.report_folder, 'CC.xml'))
+        cyclo_complex = [cyclo_complex + "," + str(int(
+            complexity.get(cyclo_complex))) for
+                         cyclo_complex in complexity if int(
+                             complexity.get(cyclo_complex)) > self.cc_limit]
+        cyclo_repport = open(os.path.join(self.report_folder,
+                                          "cyclo_failure.csv"), "w")
+        csv.writer(cyclo_repport, delimiter=',').writerows(
+            [x.split(',') for x in cyclo_complex])
+        cyclo_repport.close()
         self.validate_return(len(cyclo_complex), "Cyclomatic complexity", True)
         print("Passed Cyclomatic complexity gating")  # pragma: no mutate
         print('====================================')  # pragma: no mutate
 
     def guardrail_deadcode(self):
-        """ Function which is used to conduct the dead code analysis and gate it """
+        """ Function which is used to conduct the dead code analysis and
+         gate it """
         LOG.info("Started deadcode gate")  # pragma: no mutate
-        retval = self.call_subprocess('%s -m vulture %s  %s >%s' %
-                                      (self.python, self.list_to_str_folders(), self.dead_code_exclude(), os.path.join(
-                                          self.report_folder, 'deadcode.txt')))
+        retval = self.call_subprocess('%s -m vulture %s  %s %s'
+                                      ' --min-confidence %s >%s' %
+                                      (self.python, self.list_to_str_folders(),
+                                       self.dead_code_exclude(),
+                                       self.dead_code_whitelist,
+                                       self.min_deadcode_confidence,
+                                       os.path.join(self.report_folder,
+                                                    'deadcode.txt')))
         self.validate_return(retval, "Dead code detection ", True)
         print("Passed Dead code gating")  # pragma: no mutate
         print('====================================')  # pragma: no mutate
@@ -387,7 +463,8 @@ class Guardails(GuardrailGlobals):
             open(file_name, 'w').close()
 
     def orchestrate_guardrails(self):
-        """ Function which is used to orchestrate all the activities of guardrails """
+        """ Function which is used to orchestrate all the activities
+         of guardrails """
         self.clean_log()
         cstr = "Guardrails for python programs"
         LOG.info(cstr.center(40, '#'))  # pragma: no mutate
@@ -395,16 +472,9 @@ class Guardails(GuardrailGlobals):
         self.check_report_dir()
         if self.linting: self.guardrail_lint()
         if self.cpd: self.guardrail_jscpd()
+        if self.deadcode: self.guardrail_deadcode()
+        if self.cycloc: self.guardrail_cyclomatic_complexity()
         if self.cov:
             self.guardrail_test()
             self.guardrail_coverage()
-            self.mov_cov_report()
         if self.mutation: self.guardrail_mutation()
-        if self.deadcode: self.guardrail_deadcode()
-        if self.cycloc: self.guardrail_cyclomatic_complexity()
-
-
-if __name__ == '__main__':
-    ARGS = create_parser(sys.argv[1:])
-    GUARD_OBJ = Guardails(ARGS.path)
-    GUARD_OBJ.orchestrate_guardrails()
